@@ -13,6 +13,7 @@ import (
 	houtil "github.com/ovn-org/ovn-kubernetes/go-controller/hybrid-overlay/pkg/util"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/kube"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/types"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	"github.com/vishvananda/netlink"
@@ -38,23 +39,18 @@ type flowCacheEntry struct {
 
 // NodeController is the node hybrid overlay controller
 type NodeController struct {
-	kube        kube.Interface
 	nodeName    string
 	initialized bool
 	drMAC       net.HardwareAddr
 	drIP        net.IP
 	vxlanPort   uint16
 	// contains a map of pods to corresponding tunnels
-	tunMap      map[string]string
-	tunMapMutex sync.Mutex
-	// flow cache map of cookies to flows
 	flowCache map[string]*flowCacheEntry
 	flowMutex sync.Mutex
 	// channel to indicate we need to update flows immediately
 	flowChan chan struct{}
 
 	nodeLister listers.NodeLister
-	podLister  listers.PodLister
 }
 
 // newNodeController returns a node handler that listens for node events
@@ -63,23 +59,18 @@ type NodeController struct {
 //  1. Setting up a VXLAN gateway and hooking to the OVN gateway
 //  2. Setting back annotations about its VTEP and gateway MAC address to its own object
 func newNodeController(
-	kube kube.Interface,
+	_ kube.Interface,
 	nodeName string,
 	nodeLister listers.NodeLister,
-	podLister listers.PodLister,
 ) (nodeController, error) {
 
 	node := &NodeController{
-		kube:        kube,
-		nodeName:    nodeName,
-		vxlanPort:   uint16(config.HybridOverlay.VXLANPort),
-		tunMap:      make(map[string]string),
-		tunMapMutex: sync.Mutex{},
-		flowCache:   make(map[string]*flowCacheEntry),
-		flowMutex:   sync.Mutex{},
-		flowChan:    make(chan struct{}, 1),
-		nodeLister:  nodeLister,
-		podLister:   podLister,
+		nodeName:   nodeName,
+		vxlanPort:  uint16(config.HybridOverlay.VXLANPort),
+		flowCache:  make(map[string]*flowCacheEntry),
+		flowMutex:  sync.Mutex{},
+		flowChan:   make(chan struct{}, 1),
+		nodeLister: nodeLister,
 	}
 	return node, nil
 }
@@ -95,27 +86,16 @@ func podIPToCookie(podIP net.IP) string {
 
 // AddPod handles the pod add event
 func (n *NodeController) AddPod(pod *kapi.Pod) error {
+	// nothing to do for hostnetworked pod
+	if !util.PodWantsNetwork(pod) {
+		return nil
+	}
 	podIPs, podMAC, err := getPodDetails(pod)
 	if err != nil {
 		klog.V(5).Infof("Cleaning up hybrid overlay pod %s/%s because %v", pod.Namespace, pod.Name, err)
 		return n.DeletePod(pod)
 	}
 
-	externalGw, ok := pod.Annotations[hotypes.HybridOverlayExternalGw]
-	// validate the external gateway (if any) is a valid IP address
-	if ip := net.ParseIP(externalGw); ok && ip == nil {
-		klog.Warningf("Failed parse a valid external gateway ip address from %v: %v", externalGw, err)
-		return fmt.Errorf("failed to validate a valid external gateway ip address %s: %v", externalGw, err)
-	}
-
-	VTEP, ok := pod.Annotations[hotypes.HybridOverlayVTEP]
-	// validate the VTEP (if any) is a valid IP address
-	VTEPIP := net.ParseIP(VTEP)
-	if ok && VTEPIP == nil {
-		klog.Warningf("Failed parse a valid vtep ip address from %v: %v", VTEP, err)
-		return fmt.Errorf("failed to validate a valid vtep ip address %s: %v", VTEP, err)
-	}
-
 	// It's always safe to ignore the learn flow as we only process and add or update
 	// if the IP/MAC or Annotations have changed
 	ignoreLearn := true
@@ -146,76 +126,6 @@ func (n *NodeController) AddPod(pod *kapi.Pod) error {
 				"actions=set_field:%s->eth_src,set_field:%s->eth_dst,output:ext",
 			cookie, podIP.IP, n.drMAC.String(), podMAC))
 
-		if externalGw == "" || VTEP == "" {
-			klog.Infof("Hybrid Overlay Gateway mode not enabled for pod %s, namespace does not have hybrid"+
-				"annotations, external gw: %s, VTEP: %s", pod.Name, externalGw, VTEP)
-			n.updateFlowCacheEntry(cookie, flows, ignoreLearn)
-			continue
-		}
-
-		portMACRaw := strings.Replace(n.drMAC.String(), ":", "", -1)
-		vtepIPRaw := getIPAsHexString(VTEPIP)
-
-		// update map for tun to pod
-		n.tunMapMutex.Lock()
-		n.tunMap[podIP.IP.String()] = VTEP
-		// iterate and find all pods that belong to this VTEP and create learn actions
-		learnActions := ""
-		for pod, tun := range n.tunMap {
-			if tun == VTEP {
-				if len(learnActions) > 0 {
-					learnActions += ","
-				}
-				learnActions += fmt.Sprintf("learn("+
-					"table=20,cookie=0x%s,priority=50,"+
-					"dl_type=0x0800,nw_src=%s,"+
-					"load:NXM_NX_ARP_SHA[]->NXM_OF_ETH_DST[],"+
-					"load:0x%s->NXM_OF_ETH_SRC[],"+
-					"load:%d->NXM_NX_TUN_ID[0..31],"+
-					"load:0x%s->NXM_NX_TUN_IPV4_DST[],"+
-					"output:NXM_OF_IN_PORT[])",
-					podIPToCookie(net.ParseIP(pod)), pod, portMACRaw, hotypes.HybridOverlayVNI, vtepIPRaw)
-			}
-		}
-
-		// for arp request/response from vxlan, learn and add flow to table 20, for pod-> vxlan traffic
-		// special cookie needed here for tunnel
-		// tunnel cookie flows only contain a single flow ever, but it is updated by multiple pod adds
-		// so need proper locking around tunMap
-		// after learning actions, we need to resubmit the flow to the gw arp response table (2) so that we can respond
-		// back if this was an arp request
-		tunCookie := podIPToCookie(VTEPIP)
-		tunFlow := fmt.Sprintf("table=0,cookie=0x%s,priority=120,in_port=%s,arp,arp_spa=%s,tun_src=%s,"+
-			"actions=%s,resubmit(,2)",
-			tunCookie, extVXLANName, externalGw, VTEP, learnActions)
-		n.updateFlowCacheEntry(tunCookie, []string{tunFlow}, false)
-		n.tunMapMutex.Unlock()
-
-		// add flow to table 0 to match on incoming traffic from pods, send to table 20
-		// bypass regular Hybrid overlay for gateway mode
-		flows = append(flows,
-			fmt.Sprintf("table=0, cookie=0x%s, priority=10000,in_port=ext,ip,nw_src=%s,"+
-				"actions=goto_table:20",
-				cookie, podIP.IP))
-
-		// we need to send an ARP request to get the GW to send us a response
-		// and learn the mac, we will trigger an arp request to the gateway in table 1
-		flows = append(flows,
-			fmt.Sprintf(""+
-				"table=1,cookie=0x%s,priority=10,arp,arp_tpa=%s,"+
-				"actions="+
-				"mod_dl_dst:ff:ff:ff:ff:ff:ff,"+
-				"mod_dl_src:%s,"+
-				"load:0x1->NXM_OF_ARP_OP[],"+
-				"set_field:%s->arp_sha,"+
-				"set_field:%s->arp_spa,"+
-				"set_field:%s->arp_tpa,"+
-				"set_field:00:00:00:00:00:00->arp_tha,"+
-				"load:%d->NXM_NX_TUN_ID[0..31],"+
-				"set_field:%s->tun_dst,"+
-				"output:%s",
-				cookie, podIP.IP, n.drMAC.String(), n.drMAC.String(), n.drIP, externalGw, hotypes.HybridOverlayVNI,
-				VTEP, extVXLANName))
 		n.updateFlowCacheEntry(cookie, flows, ignoreLearn)
 	}
 	n.requestFlowSync()
@@ -225,37 +135,14 @@ func (n *NodeController) AddPod(pod *kapi.Pod) error {
 
 // DeletePod handles the pod delete event
 func (n *NodeController) DeletePod(pod *kapi.Pod) error {
+	// nothing to do for hostnetworked pods
+	if !util.PodWantsNetwork(pod) {
+		return nil
+	}
 	podIPs, _, err := getPodDetails(pod)
 	if err != nil {
 		return fmt.Errorf("error getting pod details: %v", err)
 	}
-	tunIPs := make(map[string]struct{})
-	n.tunMapMutex.Lock()
-	for _, podIP := range podIPs {
-		// need to check if any pods in the tunMap still correspond to a tunnel
-		// store the tunIP so we can delete cookie later
-		tunIPs[n.tunMap[podIP.IP.String()]] = struct{}{}
-		delete(n.tunMap, podIP.IP.String())
-	}
-	for tunIP := range tunIPs {
-		if len(tunIP) > 0 {
-			// check if any pods still belong to this tunnel so we can clean up the flow if not
-			tunStillActive := false
-			for _, tun := range n.tunMap {
-				if tunIP == tun {
-					tunStillActive = true
-					break
-				}
-			}
-			if !tunStillActive {
-				cookie := podIPToCookie(net.ParseIP(tunIP))
-				if cookie != "" {
-					n.deleteFlowsByCookie(cookie)
-				}
-			}
-		}
-	}
-	n.tunMapMutex.Unlock()
 	for _, podIP := range podIPs {
 		cookie := podIPToCookie(podIP.IP)
 		if cookie == "" {
@@ -519,9 +406,9 @@ func (n *NodeController) EnsureHybridOverlayBridge(node *kapi.Node) error {
 	if len(config.HybridOverlay.ClusterSubnets) > 0 {
 		// Add a route via the hybrid overlay port IP through the management port
 		// interface for each hybrid overlay cluster subnet
-		mgmtPortLink, err := netlink.LinkByName(util.K8sMgmtIntfName)
+		mgmtPortLink, err := netlink.LinkByName(types.K8sMgmtIntfName)
 		if err != nil {
-			return fmt.Errorf("failed to lookup link %s: %v", util.K8sMgmtIntfName, err)
+			return fmt.Errorf("failed to lookup link %s: %v", types.K8sMgmtIntfName, err)
 		}
 		mgmtPortMAC := mgmtPortLink.Attrs().HardwareAddr
 		for _, clusterEntry := range config.HybridOverlay.ClusterSubnets {
