@@ -325,14 +325,9 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 				return fmt.Errorf("failed to create default SNAT rules for gateway router %s: %v",
 					gatewayRouter, err)
 			}
-			// delete the existing lr-nat rule first otherwise gateway init fails
-			// if the external ip has changed, but the logical ip has stayed the same
-			stdout, stderr, err := util.RunOVNNbctl("--if-exists", "lr-nat-del",
-				gatewayRouter, "snat", entry.String(), "--", "lr-nat-add",
-				gatewayRouter, "snat", externalIP[0].String(), entry.String())
-			if err != nil {
-				return fmt.Errorf("failed to create default SNAT rules for gateway router %s, "+
-					"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
+			if err := util.UpdateRouterSNAT(gatewayRouter, externalIP[0], entry); err != nil {
+				return fmt.Errorf("failed to update NAT entry for pod subnet: %s, GR: %s, error: %v",
+					entry.String(), gatewayRouter, err)
 			}
 		}
 	}
