@@ -2,12 +2,12 @@ package ovn
 
 import (
 	"fmt"
-	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"net"
 	"strings"
 	"time"
 
-	hotypes "github.com/ovn-org/ovn-kubernetes/go-controller/hybrid-overlay/pkg/types"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	addressset "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/ovn/address_set"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	kapi "k8s.io/api/core/v1"
@@ -21,6 +21,7 @@ const (
 	routingExternalGWsAnnotation = "k8s.ovn.org/routing-external-gws"
 	routingNamespaceAnnotation   = "k8s.ovn.org/routing-namespaces"
 	routingNetworkAnnotation     = "k8s.ovn.org/routing-network"
+	bfdAnnotation                = "k8s.ovn.org/bfd-enabled"
 )
 
 func (oc *Controller) syncNamespaces(namespaces []interface{}) {
@@ -189,30 +190,14 @@ func (oc *Controller) AddNamespace(ns *kapi.Namespace) {
 	defer nsInfo.Unlock()
 
 	var err error
-	annotation := ns.Annotations[hotypes.HybridOverlayExternalGw]
-	if annotation != "" {
-		parsedAnnotation := net.ParseIP(annotation)
-		if parsedAnnotation == nil {
-			klog.Errorf("Could not parse hybrid overlay external gw annotation")
-		} else {
-			nsInfo.hybridOverlayExternalGW = parsedAnnotation
-		}
-	}
-	annotation = ns.Annotations[hotypes.HybridOverlayVTEP]
-	if annotation != "" {
-		parsedAnnotation := net.ParseIP(annotation)
-		if parsedAnnotation == nil {
-			klog.Errorf("Could not parse hybrid overlay VTEP annotation")
-		} else {
-			nsInfo.hybridOverlayVTEP = parsedAnnotation
-		}
-	}
-	annotation = ns.Annotations[routingExternalGWsAnnotation]
-	if annotation != "" {
-		nsInfo.routingExternalGWs, err = parseRoutingExternalGWAnnotation(annotation)
+	if annotation, ok := ns.Annotations[routingExternalGWsAnnotation]; ok {
+		nsInfo.routingExternalGWs.gws, err = parseRoutingExternalGWAnnotation(annotation)
 		if err != nil {
 			klog.Errorf(err.Error())
 		}
+		if _, ok := ns.Annotations[bfdAnnotation]; ok {
+			nsInfo.routingExternalGWs.bfdEnabled = true
+		}
 	}
 	nsInfo.addressSet, err = oc.createNamespaceAddrSetAllPods(ns.Name)
 	if err != nil {
@@ -237,13 +222,15 @@ func (oc *Controller) updateNamespace(old, newer *kapi.Namespace) {
 	}
 	defer nsInfo.Unlock()
 
-	var annotation, oldAnnotation string
-	annotation = newer.Annotations[routingExternalGWsAnnotation]
-	oldAnnotation = old.Annotations[routingExternalGWsAnnotation]
-	if annotation != oldAnnotation {
+	gwAnnotation := newer.Annotations[routingExternalGWsAnnotation]
+	oldGWAnnotation := old.Annotations[routingExternalGWsAnnotation]
+	_, newBFDEnabled := newer.Annotations[bfdAnnotation]
+	_, oldBFDEnabled := old.Annotations[bfdAnnotation]
+
+	if gwAnnotation != oldGWAnnotation || newBFDEnabled != oldBFDEnabled {
 		// if old gw annotation was empty, new one must not be empty, so we should remove any per pod SNAT
-		if oldAnnotation == "" {
-			if config.Gateway.DisableSNATMultipleGWs && (len(nsInfo.routingExternalGWs) != 0 || len(nsInfo.routingExternalPodGWs) != 0) {
+		if oldGWAnnotation == "" {
+			if config.Gateway.DisableSNATMultipleGWs && (len(nsInfo.routingExternalGWs.gws) != 0 || len(nsInfo.routingExternalPodGWs) != 0) {
 				existingPods, err := oc.watchFactory.GetPods(old.Name)
 				if err != nil {
 					klog.Errorf("Failed to get all the pods (%v)", err)
@@ -261,18 +248,18 @@ func (oc *Controller) updateNamespace(old, newer *kapi.Namespace) {
 		} else {
 			oc.deleteGWRoutesForNamespace(nsInfo)
 		}
-		exGateways, err := parseRoutingExternalGWAnnotation(annotation)
+		exGateways, err := parseRoutingExternalGWAnnotation(gwAnnotation)
 		if err != nil {
 			klog.Error(err.Error())
 		} else {
-			err = oc.addExternalGWsForNamespace(exGateways, nsInfo, old.Name)
+			err = oc.addExternalGWsForNamespace(gatewayInfo{gws: exGateways, bfdEnabled: newBFDEnabled}, nsInfo, old.Name)
 			if err != nil {
 				klog.Error(err.Error())
 			}
 		}
 		// if new annotation is empty, exgws were removed, may need to add SNAT per pod
 		// check if there are any pod gateways serving this namespace as well
-		if annotation == "" && len(nsInfo.routingExternalPodGWs) == 0 && config.Gateway.DisableSNATMultipleGWs {
+		if gwAnnotation == "" && len(nsInfo.routingExternalPodGWs) == 0 && config.Gateway.DisableSNATMultipleGWs {
 			existingPods, err := oc.watchFactory.GetPods(old.Name)
 			if err != nil {
 				klog.Errorf("Failed to get all the pods (%v)", err)
@@ -289,28 +276,6 @@ func (oc *Controller) updateNamespace(old, newer *kapi.Namespace) {
 			}
 		}
 	}
-	annotation = newer.Annotations[hotypes.HybridOverlayExternalGw]
-	if annotation != "" {
-		parsedAnnotation := net.ParseIP(annotation)
-		if parsedAnnotation == nil {
-			klog.Errorf("Could not parse hybrid overlay external gw annotation")
-		} else {
-			nsInfo.hybridOverlayExternalGW = parsedAnnotation
-		}
-	} else {
-		nsInfo.hybridOverlayExternalGW = nil
-	}
-	annotation = newer.Annotations[hotypes.HybridOverlayVTEP]
-	if annotation != "" {
-		parsedAnnotation := net.ParseIP(annotation)
-		if parsedAnnotation == nil {
-			klog.Errorf("Could not parse hybrid overlay VTEP annotation")
-		} else {
-			nsInfo.hybridOverlayVTEP = parsedAnnotation
-		}
-	} else {
-		nsInfo.hybridOverlayVTEP = nil
-	}
 	oc.multicastUpdateNamespace(newer, nsInfo)
 }
 
@@ -387,7 +352,7 @@ func (oc *Controller) createNamespaceLocked(ns string) *namespaceInfo {
 		networkPolicies:       make(map[string]*namespacePolicy),
 		podExternalRoutes:     make(map[string]map[string]string),
 		multicastEnabled:      false,
-		routingExternalPodGWs: make(map[string][]net.IP),
+		routingExternalPodGWs: make(map[string]gatewayInfo),
 	}
 	nsInfo.Lock()
 	oc.namespaces[ns] = nsInfo
@@ -425,9 +390,8 @@ func (oc *Controller) deleteNamespaceLocked(ns string) *namespaceInfo {
 	return nsInfo
 }
 
-func (oc *Controller) createNamespaceAddrSetAllPods(ns string) (AddressSet, error) {
-	// Get all the pods in the namespace and append their IP to the
-	// address_set
+func (oc *Controller) createNamespaceAddrSetAllPods(ns string) (addressset.AddressSet, error) {
+	// Get all the pods in the namespace and append their IP to the address_set
 	var ips []net.IP
 	existingPods, err := oc.watchFactory.GetPods(ns)
 	if err != nil {
