@@ -89,7 +89,7 @@ type namespaceInfo struct {
 
 	// routingExternalGWs is a slice of net.IP containing the values parsed from
 	// annotation k8s.ovn.org/routing-external-gws
-	routingExternalGWs []net.IP
+	routingExternalGWs gatewayInfo
 	// podExternalRoutes is a cache keeping the LR routes added to the GRs when
 	// the k8s.ovn.org/routing-external-gws annotation is used. The first map key
 	// is the podIP, the second the GW and the third the GR
@@ -97,7 +97,7 @@ type namespaceInfo struct {
 
 	// routingExternalPodGWs contains a map of all pods serving as exgws as well as their
 	// exgw IPs
-	routingExternalPodGWs map[string][]net.IP
+	routingExternalPodGWs map[string]gatewayInfo
 
 	// The UUID of the namespace-wide port group that contains all the pods in the namespace.
 	portGroupUUID string
@@ -610,7 +610,8 @@ func (oc *Controller) addRetryPod(pod *kapi.Pod) {
 
 func exGatewayAnnotationsChanged(oldPod, newPod *kapi.Pod) bool {
 	return oldPod.Annotations[routingNamespaceAnnotation] != newPod.Annotations[routingNamespaceAnnotation] ||
-		oldPod.Annotations[routingNetworkAnnotation] != newPod.Annotations[routingNetworkAnnotation]
+		oldPod.Annotations[routingNetworkAnnotation] != newPod.Annotations[routingNetworkAnnotation] ||
+		oldPod.Annotations[bfdAnnotation] != newPod.Annotations[bfdAnnotation]
 }
 
 // ensurePod tries to set up a pod. It returns success or failure; failure
